package com.example.garrettmoody.facerecognitionapp;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Toast;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;


public class MainActivity extends Activity {
    ArrayList<FaceNamePair> faceNamePairArrayList = new ArrayList<>();
    Uri uriSavedImage;

    public Button.OnClickListener photoGalleryListener = new Button.OnClickListener() {
        @Override
        public void onClick(View v) {
            Intent intent = new Intent(getApplicationContext(), PhotoGallery.class);
            startActivity(intent);
        }
    };

    public Button.OnClickListener cameraButtonListener = new Button.OnClickListener() {
        @Override
        public void onClick(View v) {
            Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
            File photo = null;
            try {
                photo = createTemporaryFile("picture", ".jpg");
                photo.delete();
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (photo != null) {
                uriSavedImage = Uri.fromFile(photo);
            }


            intent.putExtra(MediaStore.EXTRA_OUTPUT, uriSavedImage);
            startActivityForResult(intent, 100);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Button button = (Button) findViewById(R.id.camera_button);
        button.setOnClickListener(cameraButtonListener);

        Button photoGallery = (Button) findViewById(R.id.photoGallery);
        photoGallery.setOnClickListener(photoGalleryListener);

        List<FaceNamePair> list = loadFaceNamePairsFromFile();
        faceNamePairArrayList.addAll(list);

        ImageView faceRecIcon = (ImageView) findViewById(R.id.faceRecIcon);
        Resources res = getResources();
        Drawable drawable = res.getDrawable(R.drawable.ic_launcher);
        faceRecIcon.setImageDrawable(drawable);

    }

    public void saveFaceNamePairsToFile() {
        Context context = getApplicationContext();
        File faceNamePairsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");

        String filename = faceNamePairsFile.getPath();
        FileOutputStream fos = null;
        ObjectOutputStream out = null;
        try {
            fos = new FileOutputStream(filename);
            out = new ObjectOutputStream(fos);
            for (int i = 0; i < faceNamePairArrayList.size(); i++) {
                out.writeFloat(faceNamePairArrayList.get(i).getEyeDistance());
                out.writeInt(faceNamePairArrayList.get(i).getID());
                out.writeObject(faceNamePairArrayList.get(i).getName());
                out.writeObject(faceNamePairArrayList.get(i).getUriString());
            }

            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public List<FaceNamePair> loadFaceNamePairsFromFile() {
        Context context = getApplicationContext();
        File contactsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");
        if (!contactsFile.exists() || contactsFile.length() == 0) {
            List<FaceNamePair> emptyList = null;
            return emptyList;
        }

        List<FaceNamePair> loadedFaceNamePairs = new ArrayList<>();

        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(contactsFile);
        } catch (FileNotFoundException e) {
            Toast.makeText(getApplicationContext(), "Nothing to load.", Toast.LENGTH_LONG);
            return new ArrayList<>();
        }

        try {
            ObjectInputStream ois = new ObjectInputStream(fileInputStream);
            boolean again = true;
            while (again) {
                float eyeDistance = ois.readFloat();
                int ID = ois.readInt();
                String name = (String) ois.readObject();
                String Uri = (String) ois.readObject();
                FaceNamePair newFace = new FaceNamePair(name, eyeDistance, Uri, ID);
                loadedFaceNamePairs.add(newFace);
                if (newFace == null)
                    again = false;
            }

            ois.close();
            fileInputStream.close();
        } catch (EOFException e)
        //ignore
        {
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        }

        return loadedFaceNamePairs;
    }


    private File createTemporaryFile(String part, String ext) throws Exception {
        File tempDir = Environment.getExternalStorageDirectory();
        tempDir = new File(tempDir.getAbsolutePath() + "/.temp/");
        if (!tempDir.exists()) {
            tempDir.mkdir();
        }
        return File.createTempFile(part, ext, tempDir);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        saveFaceNamePairsToFile();
        if (faceNamePairArrayList == null || faceNamePairArrayList.size() == 0) {
            Intent intent = new Intent(this, matchNameWithFace.class);
            intent.putExtra("imageUri", uriSavedImage);
            startActivity(intent);
        } else {
            Intent intent = new Intent(this, FaceGuess.class);
            intent.putExtra("imageUri", uriSavedImage);
            startActivity(intent);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}


